/*** BRESENHAM.C ***/

typedef struct {int x, y; } t_ponto;

#define MAX_PONTOS 3000
t_ponto p[MAX_PONTOS];
int quant_pontos = 0;

/* ----------------------------------------------------------------------- */

void Reta(int x0, int y0, int x1, int y1, int cor) {

/* Desenha a reta (x0, y0) - (x1, y1) usando o algoritmo de Bresenham */

  int aux, primeiro_octante, dx, dy, incrE, incrNE,
      X0, X1, Y0, Y1, d, x, y, inclinacao_negativa;

  if (x0 > x1) {
    aux = x0; x0 = x1; x1 = aux;
    aux = y0; y0 = y1; y1 = aux;
  }

  X0 = x0; X1= x1; Y0 = y0; Y1 = y1;

  inclinacao_negativa = (y0 > y1);
  if (inclinacao_negativa) {
    aux = y0; y0 = y1; y1 = aux;
  }

  primeiro_octante = (y1 - y0 <= x1 - x0);
  if (!primeiro_octante) {
    aux = x0; x0 = y0; y0 = aux;
    aux = x1; x1 = y1; y1 = aux;
  }

  dx = x1 - x0;
  dy = y1 - y0;
  d = 2*dy - dx;
  incrE = 2*dy;
  incrNE = 2*(dy - dx);
  x = x0;
  y = y0;

  if (primeiro_octante)
    if (inclinacao_negativa)
      putpixel(x, Y1 + Y0 - y, cor);
    else
      putpixel(x, y, cor);
  else
    if (inclinacao_negativa)
      putpixel(X1 + X0 - y, x, cor);
    else
      putpixel(y, x, cor);

  while (x < x1) {

    if (d < 0) {
      d = d + incrE;
      x = x + 1;
    } else {
      d = d + incrNE;
      x = x + 1;
      y = y + 1;
    }

    if (primeiro_octante)
      if (inclinacao_negativa)
	putpixel(x, Y1 + Y0 - y, cor);
      else
	putpixel(x, y, cor);
    else
      if (inclinacao_negativa)
	putpixel(X1 + X0 - y, x, cor);
      else
	putpixel(y, x, cor);

  }
}

/* ----------------------------------------------------------------------- */

void SequenciaDePontosDaReta(int x0, int y0, int x1, int y1) {

/* Determina os pontos da reta (x0, y0) - (x1, y1) usando o algoritmo de
   Bresenham. Os pontos sao guardados na matriz de pontos global p[]. */

  int aux, primeiro_octante, dx, dy, incrE, incrNE,
      X0, X1, Y0, Y1, d, x, y, inclinacao_negativa;

  if (x0 > x1) {
    aux = x0; x0 = x1; x1 = aux;
    aux = y0; y0 = y1; y1 = aux;
  }

  X0 = x0; X1= x1; Y0 = y0; Y1 = y1;

  inclinacao_negativa = (y0 > y1);
  if (inclinacao_negativa) {
    aux = y0; y0 = y1; y1 = aux;
  }

  primeiro_octante = (y1 - y0 <= x1 - x0);
  if (!primeiro_octante) {
    aux = x0; x0 = y0; y0 = aux;
    aux = x1; x1 = y1; y1 = aux;
  }

  dx = x1 - x0;
  dy = y1 - y0;
  d = 2*dy - dx;
  incrE = 2*dy;
  incrNE = 2*(dy - dx);
  x = x0;
  y = y0;

  if (primeiro_octante)
    if (inclinacao_negativa)
      { p[quant_pontos].x = x; p[quant_pontos].y = Y0 + Y1 - y; }
    else
      { p[quant_pontos].x = x; p[quant_pontos].y = y; }
  else
    if (inclinacao_negativa)
      { p[quant_pontos].x = X0 + X1 - y; p[quant_pontos].y = x; }
    else
      { p[quant_pontos].x = y; p[quant_pontos].y = x; }
  quant_pontos++;

  while (x < x1) {

    if (d < 0) {
      d = d + incrE;
      x = x + 1;
    } else {
      d = d + incrNE;
      x = x + 1;
      y = y + 1;
    }

    if (primeiro_octante)
      if (inclinacao_negativa)
	{ p[quant_pontos].x = x; p[quant_pontos].y = Y0 + Y1 - y; }
      else
	{ p[quant_pontos].x = x; p[quant_pontos].y = y; }
    else
      if (inclinacao_negativa)
	{ p[quant_pontos].x = X0 + X1 - y; p[quant_pontos].y = x; }
      else
	{ p[quant_pontos].x = y; p[quant_pontos].y = x; }
    quant_pontos++;
  }
}

/* ----------------------------------------------------------------------- */

/*** FIM DE BRESENHAM.C ***/